/*
 * {tmp112.c}
 *
 * {Functions}
 *
 *  Copyright 2011-2015 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user who
  downloaded the software, his/her employer (which must be your employer) and
  Texas Instruments Incorporated (the "License"). You may not use this Software
  unless you agree to abide by the terms of the License. The License limits your
  use, and you acknowledge, that the Software may not be modified, copied or
  distributed unless embedded on a Texas Instruments microcontroller which is
  integrated into your product. Other than for the foregoing purpose, you may
  not use, reproduce, copy, prepare derivative works of, modify, distribute,
  perform, display or sell this Software and/or its documentation for any
  purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
  INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
  EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
  LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
  DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
  TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
  LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/



#include "tmp112.h"
#include <msp430f5529.h>




unsigned char *PTxData;                     // Pointer to TX data
unsigned char TXByteCtr;


//------------------------------------------------------------------------------
// void USCB1_I2C_setup(unsigned char slave_address,
//                          unsigned char prescale)
//
// Function initializes the USB1 module for master-control operation.
//
// IN:   unsigned char slave_address   =>  Slave Address
//       unsigned char prescale        =>  SCL clock adjustment
//------------------------------------------------------------------------------
void USCB1_I2C_setup(unsigned char slave_address, unsigned char prescale)
{
  P4SEL |= 0x06;//SDA_PIN + SCL_PIN;          // Assign I2C pins to USCI_B1
  UCB1CTL1 |= UCSWRST;                  // put eUSCI_B in reset state
  UCB1CTL0 = UCMST + UCMODE_3 + UCSYNC; // I2C Master, synchronous mode
  UCB1CTL1 = UCSSEL_2 + UCSWRST;        // Use SMCLK
  UCB1BR0 = prescale;                   // fSCL = SMCLK/240 = ~100kHz
  UCB1BR1 = 0;
  //UCB1I2CSA = slave_address;          // address slave
  UCB1CTL1 &= ~UCSWRST;                 // eUSCI_B in operational state
  UCB1IE |= UCTXIE;                   // enable TX-interrupt
//  GIE;                                // general interrupt enable

  __delay_cycles(1000);                 // At 25Mhz, this delay is a must
                                        // .. before device can be accessed
  while (UCB1STAT & UCBBUSY);           // wait for bus to be free
}


//------------------------------------------------------------------------------
// unsigned int ReadTempAtUSCB1_I2C(unsigned char slave_address)
//
// This function is used for getting the temperature from TMP102
//
// IN:   unsigned char slave_address  =>  Slave Address
// OUT:  16bit register with temperature information, as last 4 bits only 0s, data is shifted to the right
//
//------------------------------------------------------------------------------
unsigned int ReadTempAtUSCB1_I2C(unsigned char slave_address)
{
  unsigned int returnValue[2];
  unsigned int ui16ReturnData = 0;
  UCB1I2CSA = 0x48;//slave_address;                // set slave address
  __disable_interrupt();

    // Transmit address RW=0
    while (UCB1CTL1 & UCTXSTP);             // Ensure stop condition sent
    UCB1CTL1 |= UCTR + UCTXSTT;             // I2C TX, start condition
    UCB1TXBUF = 0x00;                       // Load TX buffer
    while (UCB1CTL1 & UCTXIFG);             // Ensure TX sent
    UCB1CTL1 |= UCTXSTP;                    // stop condition
    while (UCB1CTL1 & UCTXSTP);             // Ensure stop condition sent

    // Transmit address RW=1 (automatic by MSP)
    UCB1CTL1 |= UCTR + UCTXSTT;             // I2C TX, start condition
    UCB1CTL1 &= ~UCTR;                      // Change I2C to RX
    UCB1TXBUF = 0x00;                       // Load TX buffer - dummy to read byte
    while (~UCB1IFG & UCRXIFG);             // Ensure RX is completed
    returnValue[0] = UCB1RXBUF;             // Get RX data - 1st byte
    UCB1CTL1 &= ~UCTXNACK;                  // Generate ACK
    UCB1TXBUF = 0x00;                       // Load TX buffer - dummy to read byte
    while (~UCB1IFG & UCRXIFG);             // Ensure RX is completed
    returnValue[1] = UCB1RXBUF;             // Get RX data - 2nd byte (not needed for most cases)
    UCB1CTL1 |= UCTXSTP;                    // stop condition
    while (UCB1CTL1 & UCTXSTP);             // Ensure stop condition got sent

    __enable_interrupt();

    ui16ReturnData = (returnValue[1]) | (returnValue[0] << 8);
    return ui16ReturnData= ui16ReturnData >> 4; // First byte is enough in most cases provided that
                                            // .. negative temperature will not be measured

}

//------------------------------------------------------------------------------
// void ConfigsensorAtUSCB1_I2C(unsigned char slave_address)
//
// This function is used for configuring the config and TLOW AND THIGH registers from TMP112
//
//
//------------------------------------------------------------------------------
void ConfigsensorAtUSCB1_I2C(unsigned char slave_address)
{
  unsigned int i;

  UCB1I2CSA = 0x48;//slave_address;                // set slave address

//  __disable_interrupt();

  	 for(i=0;i<10;i++);                      // Delay required between transaction
     PTxData = (unsigned char *)TxData;      // TX array start address
                                             // Place breakpoint here to see each
                                             // transmit operation.
     TXByteCtr = 3;              // Load TX byte counter

     UCB1CTL1 |= UCTR + UCTXSTT;             // I2C TX, start condition
    //__enable_interrupt();
     __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0, enable interrupts
     __no_operation();                       // Remain in LPM0 until all data
                                             // is TX'd
     while (UCB1CTL1 & UCTXSTP);             // Ensure stop condition got sent

}




//------------------------------------------------------------------------------
// The USCIAB1TX_ISR is structured such that it can be used to transmit any
// number of bytes by pre-loading TXByteCtr with the byte count. Also, TXData
// points to the next byte to transmit.
//------------------------------------------------------------------------------
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector = USCI_B1_VECTOR
__interrupt void USCI_B1_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(USCI_B1_VECTOR))) USCI_B1_ISR (void)
#else
#error Compiler not supported!
#endif
{
  switch(__even_in_range(UCB1IV,12))
  {
  case  0: break;                           // Vector  0: No interrupts
  case  2: break;                           // Vector  2: ALIFG
  case  4: break;                           // Vector  4: NACKIFG
  case  6: break;                           // Vector  6: STTIFG
  case  8: break;                           // Vector  8: STPIFG
  case 10: break;                           // Vector 10: RXIFG
  case 12:                                  // Vector 12: TXIFG
    if (TXByteCtr)                          // Check TX byte counter
    {
      UCB1TXBUF = *PTxData++;               // Load TX buffer
      TXByteCtr--;                          // Decrement TX byte counter
    }
    else
    {
      UCB1CTL1 |= UCTXSTP;                  // I2C stop condition
      UCB1IFG &= ~UCTXIFG;                  // Clear USCI_B0 TX int flag
      __bic_SR_register_on_exit(LPM0_bits); // Exit LPM0
    }
  default: break;
  }
}















